"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.S3Helper = void 0;
const S3 = require("@aws-sdk/client-s3");
const s3_request_presigner_1 = require("@aws-sdk/s3-request-presigner");
const typescript_helper_functions_1 = require("typescript-helper-functions");
const stream_1 = require("stream");
/**
 * S3 Helper
 */
class S3Helper extends typescript_helper_functions_1.BaseClass {
    /**
     * Create new instance of S3Helper
     * @param logger {ILogger} Logger
     * @param repository {S3.S3} A new repository will be created if not supplied
     * @param options {S3.S3ClientConfig} Only needed if a repository is supplied
     */
    constructor(logger, repository, options) {
        super(logger);
        this.Options = this.ObjectOperations.IsNullOrEmpty(options) ? { region: 'us-east-1' } : options;
        this.Repository = repository || new S3.S3(this.Options);
    }
    CopyObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.CopyObjectAsync.name}`;
            this.LogHelper.LogInputs(action, { sourceBucket, sourceKey, destinationBucket, destinationKey });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(sourceBucket)) {
                throw new Error(`[${action}]-Must supply sourceBucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(sourceKey)) {
                throw new Error(`[${action}]-Must supply sourceKey`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(destinationBucket)) {
                throw new Error(`[${action}]-Must supply destinationBucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(destinationKey)) {
                throw new Error(`[${action}]-Must supply destinationKey`);
            }
            // create params object
            const params = {
                Bucket: destinationBucket,
                CopySource: `${sourceBucket}/${sourceKey}`,
                Key: destinationKey,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.copyObject(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    CreateBucketAsync(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.CreateBucketAsync.name}`;
            this.LogHelper.LogInputs(action, { name });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(name)) {
                throw new Error(`[${action}]-Must supply name`);
            }
            // create params object
            const params = {
                Bucket: name,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.createBucket(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    DeleteBucketAsync(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteBucketAsync.name}`;
            this.LogHelper.LogInputs(action, { name });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(name)) {
                throw new Error(`[${action}]-Must supply name`);
            }
            // create params object
            const params = {
                Bucket: name,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteBucket(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    DeleteObjectAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteObjectAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // create params object
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteObject(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    DeleteObjectsAsync(bucket, keys) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteObjectsAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, keys });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (!keys || keys.length === 0) {
                throw new Error(`[${action}]-Must supply at least one key`);
            }
            // create array of ObjectIdentifiers
            const keysArray = [];
            for (const key of keys) {
                keysArray.push({ Key: key });
            }
            // create params object
            const params = {
                Bucket: bucket,
                Delete: { Objects: keysArray },
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteObjects(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    DeleteObjectTagsAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteObjectTagsAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            const response = yield this.Repository.deleteObjectTagging(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    GetBucketMetadataAsync(bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetBucketMetadataAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            const params = {
                Bucket: bucket,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.headBucket(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    GetObjectAsJsonAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield this.GetObjectContentsAsync(bucket, key);
            const json = data ? data.toString() : '';
            return JSON.parse(json);
        });
    }
    GetObjectAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetObjectAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // create params object
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.getObject(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    GetObjectContentsAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = (yield this.GetObjectAsync(bucket, key)).Body;
            return data ? data : undefined;
        });
    }
    GetObjectMetadataAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetObjectMetadataAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.headObject(params);
            this.LogHelper.LogResponse(action, response);
            return response.Metadata;
        });
    }
    GetObjectTagsAsync(bucket, key) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetObjectTagsAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            const response = yield this.Repository.getObjectTagging(params);
            this.LogHelper.LogResponse(action, response);
            return (_a = response.TagSet) !== null && _a !== void 0 ? _a : [];
        });
    }
    GetSignedUrlDownload(bucket, key, timeoutInMinutes = 5) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetSignedUrlDownload.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, timeoutInMinutes });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            const client = new S3.S3Client(this.Options);
            const operation = new S3.GetObjectCommand(params);
            const response = yield s3_request_presigner_1.getSignedUrl(client, operation, { expiresIn: timeoutInMinutes * 60 });
            this.LogHelper.LogResponse(action, { response });
            return response;
        });
    }
    GetSignedUrlUpload(bucket, key, timeoutInMinutes = 5, acl) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetSignedUrlUpload.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, acl, timeoutInMinutes });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
                ACL: acl,
            };
            this.LogHelper.LogRequest(action, params);
            const client = new S3.S3Client(this.Options);
            const operation = new S3.PutObjectCommand(params);
            const response = yield s3_request_presigner_1.getSignedUrl(client, operation, { expiresIn: timeoutInMinutes * 60 });
            this.LogHelper.LogResponse(action, { response });
            return response;
        });
    }
    MoveObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const copyResult = yield this.CopyObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey);
            if (!this.ObjectOperations.IsNullOrEmpty(copyResult)) {
                yield this.DeleteObjectAsync(sourceBucket, sourceKey);
            }
        });
    }
    MultipartUploadCompleteAsync(bucket, key, uploadId) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.MultipartUploadCompleteAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, uploadId });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(uploadId)) {
                throw new Error(`[${action}]-Must supply uploadId`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
                UploadId: uploadId,
            };
            this.LogHelper.LogRequest(action, params);
            const response = yield this.Repository.completeMultipartUpload(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    MultipartUploadStartAsync(bucket, key, acl) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.MultipartUploadStartAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, acl });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
                ACL: acl,
            };
            this.LogHelper.LogRequest(action, params);
            const response = yield this.Repository.createMultipartUpload(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    MultipartUploadUploadPartAsync(bucket, key, uploadId, uploadPart, contents) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.MultipartUploadUploadPartAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, uploadId, uploadPart });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(uploadId)) {
                throw new Error(`[${action}]-Must supply uploadId`);
            }
            if (uploadPart === 0) {
                throw new Error(`[${action}]-Cannot be zero - uploadId`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(contents)) {
                throw new Error(`[${action}]-Must supply contents`);
            }
            const bytes = (new TextEncoder()).encode(contents);
            const tooSmall = bytes.length < 5000000;
            const tooLarge = bytes.length > 10000000000;
            if (tooSmall || tooLarge) {
                throw new Error(`[${action}]-Part size must be between 5 MB and 10 GB`);
            }
            const readable = new stream_1.Readable();
            readable.push(contents);
            readable.push(null);
            const params = {
                Body: readable,
                Bucket: bucket,
                Key: key,
                PartNumber: uploadPart,
                UploadId: uploadId,
            };
            this.LogHelper.LogRequest(action, params);
            const response = yield this.Repository.uploadPart(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    PutObjectAsync(bucket, key, contents, acl, encoding) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.PutObjectAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, acl });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // set defaults
            if (this.ObjectOperations.IsNullOrWhitespace(acl)) {
                acl = 'bucket-owner-full-control';
            }
            if (this.ObjectOperations.IsNullOrWhitespace(encoding)) {
                encoding = 'utf-8';
            }
            let body = new stream_1.Readable();
            ;
            switch (typeof (contents)) {
                case 'undefined':
                    throw new Error(`[${action}]-Must supply contents`);
                case 'string':
                    body.push(contents);
                    body.push(null);
                    break;
                default:
                    body = contents;
                    break;
            }
            // create params object
            const params = {
                ACL: acl,
                Body: body,
                Bucket: bucket,
                ContentEncoding: encoding,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.putObject(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    SetObjectTagAsync(bucket, key, tagName, tagValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.SetObjectTagAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, tagName, tagValue });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(tagName)) {
                throw new Error(`[${action}]-Must supply tagName`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(tagValue)) {
                throw new Error(`[${action}]-Must supply tagValue`);
            }
            // get the original
            const tags = yield this.GetObjectTagsAsync(bucket, key);
            // find if it exists
            const tag = tags.find((t) => t.Key === tagName);
            if (tag) {
                tag.Value = tagValue;
            }
            else {
                tags.push({ Key: tagName, Value: tagValue });
            }
            return this.SetObjectTagsAsync(bucket, key, tags);
        });
    }
    SetObjectTagsAsync(bucket, key, tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.SetObjectTagsAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, tags });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            if (this.ObjectOperations.IsNullOrEmpty(tags)) {
                throw new Error(`[${action}]-Must supply tags`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
                Tagging: { TagSet: tags },
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.putObjectTagging(params);
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
}
exports.S3Helper = S3Helper;
//# sourceMappingURL=helper.js.map